class ResourceNotActive(Exception):
    """
    Raised when specified resource is not active
    """

class ResourceNotFound(Exception):
    """
    Raised when specified resource is not found
    """
